/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include "gtk_widget.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "identifier.h"

static _gtk_widget *buffer;
static int size;
static int maxsize;

/* -1 error, a gtk_widget if ok */
gtk_widget new_gtk_widget(void)
{
  if (size == maxsize) {
    _gtk_widget *new;
    int ns = maxsize + 64;
    if (!maxsize)
      new = malloc(sizeof(_gtk_widget) * ns);
    else
      new = realloc(buffer, sizeof(_gtk_widget) * ns);
    if (!new) {
      fprintf(stderr, "out of memory\n");
      return -1;
    }
    buffer = new;
    maxsize = ns;
  }

  memset(&buffer[size], -1, sizeof(_gtk_widget));

  buffer[size].signals = new_list();
  if (buffer[size].signals == -1) return -1;

  size++;
  return size-1;
}

void close_gtk_widget(void)
{
  if (!maxsize) return;
  free(buffer);
  buffer = 0;
  size = maxsize = 0;
}

static int name;

/* -1 error, an identifier if ok */
identifier new_name(void)
{
  char s[32];
  sprintf(s, "_%d", name);
  name++;
  return new_identifier(s);
}

void gtk_widget_copy(gtk_widget from, gtk_widget to)
{
  memcpy(&buffer[to], &buffer[from], sizeof(_gtk_widget));
}

_gtk_widget *get_gtk_widget(widget w)
{
  return &buffer[w];
}

/* -1 not found */
gtk_widget find_gtk_widget_by_name(const char *ii)
{
  int i;

  {
    static int rew=0;
    if (!rew) {
      fprintf(stderr, "%s:%d:%s: rewrite, too slow\n",
              __FILE__, __LINE__, __FUNCTION__);
      rew=1;
    }
  }

  for (i = 0; i < size; i++)
    if (!strcmp(ii, get_identifier(WIDGET_NAME(i))))
      return i;

  return -1;
}
