/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include <string.h>

#include "widget.h"
#include "generate.h"
#include "files.h"
#include "widget.h"
#include "keywords.h"
#include "code.h"
#include "globals.h"

extern widget_driver file_widget_driver;

/* -1 error, 0 ok */
static int first_pass_translation(widget w, widget_element e)
{
  gtk_widget file;
  list l;
  int n;
  int i;

  if (list_size(widget_element_pack(w, e))) {
    fprintf(stderr, "packed file is not allowed (file %s)\n",
            get_identifier(widget_element_name(w, e)));
    return -1;
  }

  l = widget_element_arguments(w, e);
  if (list_size(l) != 2 ||
    (list_get_int(l, 0) != WA_STRING && list_get_int(l, 0) != WA_IDENTIFIER)) {
    fprintf(stderr,
            "file %s not properly declared. You must do something like:\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "  file %s(\"file's title\");\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "or\n");
    fprintf(stderr, "  file %s(text);\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "where 'text' is a NULLZ char * argument\n");
    return -1;
  }

  file = new_gtk_widget();
  if (file == -1) return -1;

  if (widget_add_gtk_widget(w, file) == -1)
    return -1;

  widget_element_set_gtk_widget(w, e, file);

  WIDGET_TYPE(file) = WIDGET_FILE;
  WIDGET_NAME(file) = widget_element_name(w, e);
  WIDGET_DRIVER(file) = &file_widget_driver;
  if (!default_local)
    SET_GLOBAL(file, 1);
  if (list_get_int(l, 0) == WA_STRING)
    FILE_TITLE_TYPE(file) = FILE_TITLE_STRING;
  else
    FILE_TITLE_TYPE(file) = FILE_TITLE_VARIABLE;
  FILE_TITLE(file) = list_get_int(l, 1);

  /* properties */
  l = widget_element_properties(w, e);
  n = list_size(l);
  for (i=0; i<n; i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    int type = list_get_int(l, i+1);

    switch (id) {
      CASE_PROPERTY_WIDTH(file)
      CASE_PROPERTY_HEIGHT(file)
      CASE_PROPERTY_BORDER(file, l, i)
      CASE_PROPERTY_GROW(file, l, i)
      CASE_PROPERTY_PADDING(file, l, i)
      CASE_PROPERTY_GLOBAL(file)
      CASE_PROPERTY_FOREGROUND_AND_BACKGROUND(file)
      CASE_PROPERTY_SCROLL(file)
      CASE_PROPERTY_HSCROLL(file)
      CASE_PROPERTY_VSCROLL(file)
      CASE_PROPERTY_OK(file)
      CASE_PROPERTY_CANCEL(file)
      CASE_PROPERTY_FILE_POSITION(file)
      CASE_PROPERTY_DELETE(file)
      CASE_PROPERTY_DEFAULT
    }
  }

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int second_pass_translation(widget w, widget_element e)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int populate_properties(gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_struct(gtk_widget gw)
{
  if (!IS_GLOBAL(gw)) return 0;

  if (hide_gtk)
    PUT_H("  /* GtkWidget* */ void *");
  else
    PUT_H("  GtkWidget *");
  PUT_H(get_identifier(WIDGET_NAME(gw)));
  PUT_H(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_signal_handlers(widget w, gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);
  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    switch (id) {
      CASE_GENERATE_SIGNAL_HANDLER_FILE_OK
      CASE_GENERATE_SIGNAL_HANDLER_FILE_CANCEL
      CASE_GENERATE_SIGNAL_HANDLER_DELETE
      CASE_GENERATE_SIGNAL_HANDLER_DEFAULT
    }
  }
  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_declaration(gtk_widget gw)
{
  if (IS_GLOBAL(gw)) return 0;

  PUT_C("  GtkWidget *");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_instanciation(gtk_widget gw)
{
  PUT_C("  ");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(" = gtk_file_selection_new(");
  if (FILE_TITLE_TYPE(gw) == FILE_TITLE_STRING) {
    PUT_C("\"");
    PUT_C(get_string(FILE_TITLE(gw)));
    PUT_C("\"");
  } else
    PUT_C(get_identifier(FILE_TITLE(gw)));
  PUT_C(");\n");

  GENERATE_WIDTH_AND_HEIGHT
  GENERATE_BORDER_WIDTH
  GENERATE_FOREGROUND_AND_BACKGROUND
  GENERATE_FILE_POSITION

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int generate_pack(gtk_widget gw)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int generate_post_pack(gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_signals(gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);

  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);

    switch (id) {
      CASE_GENERATE_SIGNAL_OK
      CASE_GENERATE_SIGNAL_CANCEL
      CASE_GENERATE_SIGNAL_DELETE
      CASE_GENERATE_SIGNAL_DEFAULT
    }
  }
  if (list_size(l)) PUT_C("\n");

  return 0;
}

widget_driver file_widget_driver = {
  .name = "file",
  .first_pass_translation = first_pass_translation,
  .second_pass_translation = 0,
  .populate_properties = 0,
  .generate_struct = generate_struct,
  .generate_signal_handlers = generate_signal_handlers,
  .generate_variable_declaration = generate_variable_declaration,
  .generate_variable_instanciation = generate_variable_instanciation,
  .generate_pack = 0,
  .generate_post_pack = 0,
  .generate_signals = generate_signals
};
