/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include <string.h>

#include "widget.h"
#include "generate.h"
#include "files.h"
#include "gtk_widget.h"
#include "keywords.h"
#include "code.h"
#include "globals.h"

extern widget_driver image_widget_driver;

/* -1 error, 0 ok */
static int first_pass_translation(widget w, widget_element e)
{
  gtk_widget image;
  list l;
  int i;
  int n;
  int default_source;

  if (list_size(widget_element_pack(w, e))) {
    fprintf(stderr, "packed image are illegal (image %s)\n",
            get_identifier(widget_element_name(w, e)));
    return -1;
  }

  l = widget_element_arguments(w, e);
  if (list_size(l) != 2 ||
      (list_get_int(l, 0) != WA_STRING &&
       list_get_int(l, 0) != WA_IDENTIFIER)) {
    fprintf(stderr,
            "image %s not properly declared. You must do something like:\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "  image %s(\"image file name\");\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "or\n");
    fprintf(stderr, "  image %s(image_file_name_variable);\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "or\n");
    fprintf(stderr, "  image %s(image_in_memory_variable);\n",
            get_identifier(widget_element_name(w, e)));
    return -1;
  }

  image = new_gtk_widget();
  if (image == -1) return -1;

  if (widget_add_gtk_widget(w, image) == -1)
    return -1;

  widget_element_set_gtk_widget(w, e, image);

  WIDGET_TYPE(image) = IMAGE;
  WIDGET_NAME(image) = widget_element_name(w, e);
  WIDGET_DRIVER(image) = &image_widget_driver;
  if (!default_local)
    SET_GLOBAL(image, 1);
  if (list_get_int(l, 0) == WA_STRING)
    default_source = IMAGE_SOURCE_TYPE(image) = IMAGE_FILE_STRING;
  else
    default_source = IMAGE_SOURCE_TYPE(image) = IMAGE_FILE_IDENTIFIER;
  IMAGE_SOURCE(image) = list_get_int(l, 1);
  IMAGE_HALIGN(image) = -1;
  IMAGE_VALIGN(image) = -1;

  /* properties */
  l = widget_element_properties(w, e);
  n = list_size(l);
  for (i=0; i<n; i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    int type = list_get_int(l, i+1);

    switch (id) {
      CASE_PROPERTY_IMAGE_SOURCE(image)
      CASE_PROPERTY_BORDER(image, l, i)
      CASE_PROPERTY_WIDTH(image)
      CASE_PROPERTY_HEIGHT(image)
      CASE_PROPERTY_GROW(image, l, i)
      CASE_PROPERTY_PADDING(image, l, i)
      CASE_PROPERTY_HALIGN(IMAGE, image)
      CASE_PROPERTY_VALIGN(IMAGE, image)
      CASE_PROPERTY_GLOBAL(image)
      CASE_PROPERTY_SCROLL(image)
      CASE_PROPERTY_HSCROLL(image)
      CASE_PROPERTY_VSCROLL(image)
      CASE_PROPERTY_DEFAULT
    }
  }

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int second_pass_translation(widget w, widget_element e)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int populate_properties(gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_struct(gtk_widget gw)
{
  if (!IS_GLOBAL(gw)) return 0;

  if (hide_gtk)
    PUT_H("  /* GtkWidget* */ void *");
  else
    PUT_H("  GtkWidget *");
  PUT_H(get_identifier(WIDGET_NAME(gw)));
  PUT_H(";\n");

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int generate_signal_handlers(widget w, gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_variable_declaration(gtk_widget gw)
{
  if (IS_GLOBAL(gw)) return 0;

  PUT_C("  GtkWidget *");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_instanciation(gtk_widget gw)
{
  PUT_C("  ");
  if (IMAGE_SOURCE_TYPE(gw) == IMAGE_MEMORY) {
    PUT_C("{\n    GdkPixbuf *_pix = gdk_pixbuf_new_from_inline(-1, ");
    PUT_C(get_identifier(IMAGE_SOURCE(gw)));
    PUT_C(", FALSE, NULL);\n    ");
    if (IS_GLOBAL(gw)) PUT_C("this->");
    PUT_C(get_identifier(WIDGET_NAME(gw)));
    PUT_C(" = gtk_image_new_from_pixbuf(_pix);\n");
    PUT_C("    g_object_unref(_pix);\n");
    PUT_C("  }\n");
  } else {
    if (IS_GLOBAL(gw)) PUT_C("this->");
    PUT_C(get_identifier(WIDGET_NAME(gw)));
    PUT_C(" = gtk_image_new_from_file(");
    if (IMAGE_SOURCE_TYPE(gw) == IMAGE_FILE_STRING) {
      PUT_C("\"");
      PUT_C(get_string(IMAGE_SOURCE(gw)));
      PUT_C("\"");
    } else
      PUT_C(get_identifier(IMAGE_SOURCE(gw)));
    PUT_C(");\n");
  }

  GENERATE_BORDER_WIDTH_USING_PADDING
  GENERATE_WIDTH_AND_HEIGHT
  GENERATE_HALIGN_AND_VALIGN(IMAGE)

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int generate_pack(gtk_widget gw)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int generate_signals(gtk_widget gw)
{
  return 0;
}
#endif

widget_driver image_widget_driver = {
  .name = "image",
  .first_pass_translation = first_pass_translation,
  .second_pass_translation = 0,
  .populate_properties = 0,
  .generate_struct = generate_struct,
  .generate_signal_handlers = 0,
  .generate_variable_declaration = generate_variable_declaration,
  .generate_variable_instanciation = generate_variable_instanciation,
  .generate_pack = 0,
  .generate_post_pack = 0,
  .generate_signals = 0
};
