/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include <string.h>

#include "widget.h"
#include "generate.h"
#include "files.h"
#include "widget.h"
#include "keywords.h"
#include "code.h"
#include "globals.h"

extern widget_driver label_widget_driver;

/* -1 error, 0 ok */
static int first_pass_translation(widget w, widget_element e)
{
  gtk_widget label;
  list l;
  int i;
  int n;

  if (list_size(widget_element_pack(w, e))) {
    fprintf(stderr, "packed label are illegal (label %s)\n",
            get_identifier(widget_element_name(w, e)));
    return -1;
  }

  l = widget_element_arguments(w, e);
  if (list_size(l) != 2 ||
    (list_get_int(l, 0) != WA_STRING && list_get_int(l, 0) != WA_IDENTIFIER)) {
    fprintf(stderr,
            "label %s not properly declared. You must do something like:\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "  label %s(\"label's text\");\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "or\n");
    fprintf(stderr, "  label %s(text);\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "where 'text' is a NULLZ char * argument\n");
    return -1;
  }

  label = new_gtk_widget();
  if (label == -1) return -1;

  if (widget_add_gtk_widget(w, label) == -1)
    return -1;

  widget_element_set_gtk_widget(w, e, label);

  WIDGET_TYPE(label) = LABEL;
  WIDGET_NAME(label) = widget_element_name(w, e);
  WIDGET_DRIVER(label) = &label_widget_driver;
  if (!default_local)
    SET_GLOBAL(label, 1);
  if (list_get_int(l, 0) == WA_STRING)
    LABEL_TYPE(label) = LABEL_STRING;
  else
    LABEL_TYPE(label) = LABEL_VARIABLE;
  LABEL_TEXT(label) = list_get_int(l, 1);
  LABEL_HALIGN(label) = -1;
  LABEL_VALIGN(label) = -1;

  /* properties */
  l = widget_element_properties(w, e);
  n = list_size(l);
  for (i=0; i<n; i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    int type = list_get_int(l, i+1);

    switch (id) {
      CASE_PROPERTY_BORDER(label, l, i)
      CASE_PROPERTY_WIDTH(label)
      CASE_PROPERTY_HEIGHT(label)
      CASE_PROPERTY_GROW(label, l, i)
      CASE_PROPERTY_PADDING(label, l, i)
      CASE_PROPERTY_HALIGN(LABEL, label)
      CASE_PROPERTY_VALIGN(LABEL, label)
      CASE_PROPERTY_SCROLL(label)
      CASE_PROPERTY_HSCROLL(label)
      CASE_PROPERTY_VSCROLL(label)
      CASE_PROPERTY_GLOBAL(label)
      CASE_PROPERTY_DEFAULT
    }
  }

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int second_pass_translation(widget w, widget_element e)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int populate_properties(gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_struct(gtk_widget gw)
{
  if (!IS_GLOBAL(gw)) return 0;

  if (hide_gtk)
    PUT_H("  /* GtkWidget* */ void *");
  else
    PUT_H("  GtkWidget *");
  PUT_H(get_identifier(WIDGET_NAME(gw)));
  PUT_H(";\n");

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int generate_signal_handlers(widget w, gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_variable_declaration(gtk_widget gw)
{
  if (IS_GLOBAL(gw)) return 0;

  PUT_C("  GtkWidget *");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_instanciation(gtk_widget gw)
{
  PUT_C("  ");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(" = gtk_label_new(0);\n");
  PUT_C("  gtk_label_set_markup_with_mnemonic(GTK_LABEL(");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C("), ");
  if (LABEL_TYPE(gw) == LABEL_STRING) {
    PUT_C("\"");
    PUT_C(get_string(LABEL_TEXT(gw)));
    PUT_C("\"");
  } else {
    PUT_C(get_identifier(LABEL_TEXT(gw)));
  }
  PUT_C(");\n");

  GENERATE_BORDER_WIDTH_USING_PADDING
  GENERATE_WIDTH_AND_HEIGHT
  GENERATE_HALIGN_AND_VALIGN(LABEL)
  GENERATE_FONT
  GENERATE_FOREGROUND_AND_BACKGROUND

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int generate_pack(gtk_widget gw)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int generate_signals(gtk_widget gw)
{
  return 0;
}
#endif

widget_driver label_widget_driver = {
  .name = "label",
  .first_pass_translation = first_pass_translation,
  .second_pass_translation = 0,
  .populate_properties = 0,
  .generate_struct = generate_struct,
  .generate_signal_handlers = 0,
  .generate_variable_declaration = generate_variable_declaration,
  .generate_variable_instanciation = generate_variable_instanciation,
  .generate_pack = 0,
  .generate_post_pack = 0,
  .generate_signals = 0
};
