/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include <string.h>

#include "widget.h"
#include "generate.h"
#include "files.h"
#include "keywords.h"
#include "code.h"
#include "globals.h"

#if 0
/* -1 error, 0 ok */
static int first_pass_translation(widget w, widget_element e)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int second_pass_translation(widget w, widget_element e)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int populate_properties(gtk_widget gw)
{
  int has_to_populate = 0;
  gtk_widget sw = MENU_ITEM_CHILD(gw);
  widget_driver *sd = WIDGET_DRIVER(sw);

  POPULATE_PROPERTIES

  if (has_to_populate)
    if (sd->populate_properties)
      if (sd->populate_properties(sw) == -1) return -1;

  /* populate the submenu too */
  sw = MENU_ITEM_SUBMENU(gw);
  if (sw == -1)
    return 0;
  sd = WIDGET_DRIVER(sw);
  has_to_populate = 0;

  POPULATE_PROPERTIES

  if (has_to_populate)
    if (sd->populate_properties)
      if (sd->populate_properties(sw) == -1) return -1;

  return 0;
}

/* -1 error, 0 ok */
static int generate_struct(gtk_widget gw)
{
  if (!IS_GLOBAL(gw)) return 0;

  if (hide_gtk)
    PUT_H("  /* GtkWidget* */ void *");
  else
    PUT_H("  GtkWidget *");
  PUT_H(get_identifier(WIDGET_NAME(gw)));
  PUT_H(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_signal_handlers(widget w, gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);
  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    switch (id) {
      CASE_GENERATE_SIGNAL_HANDLER_MENU_ITEM_ACTIVATE
      CASE_GENERATE_SIGNAL_HANDLER_DEFAULT
    }
  }
  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_declaration(gtk_widget gw)
{
  if (IS_GLOBAL(gw)) return 0;

  PUT_C("  GtkWidget *");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_instanciation(gtk_widget gw)
{
  PUT_C("  ");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  switch (MENU_ITEM_TYPE(gw)) {
  case MENU_ITEM_NORMAL:
    PUT_C(" = gtk_menu_item_new();\n");
    break;
  case MENU_ITEM_CHECK:
    PUT_C(" = gtk_check_menu_item_new();\n");
    break;
  case MENU_ITEM_RADIO:
    PUT_C(" = gtk_radio_menu_item_new(NULL);\n");
    break;
  }

  if (MENU_ITEM_TYPE(gw) == MENU_ITEM_NORMAL && IS_ACTIVE(gw)) {
    fprintf(stderr,
   "warning: menu item '%s' is not 'check' or 'radio', 'active' meaningless\n",
            get_identifier(WIDGET_NAME(gw)));
  }

  GENERATE_FOREGROUND_AND_BACKGROUND

  return 0;
}

/* -1 error, 0 ok */
static int generate_pack(gtk_widget gw)
{
  PUT_C("  gtk_container_add(GTK_CONTAINER(");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C("), ");
  if (hide_gtk)
    PUT_C("(GtkWidget *)");
  if (IS_GLOBAL(MENU_ITEM_CHILD(gw))) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(MENU_ITEM_CHILD(gw))));
  PUT_C(");\n");
  if (MENU_ITEM_SUBMENU(gw) != -1) {
    PUT_C("  gtk_menu_item_set_submenu(GTK_MENU_ITEM(");
    if (IS_GLOBAL(gw)) PUT_C("this->");
    PUT_C(get_identifier(WIDGET_NAME(gw)));
    PUT_C("), ");
    if (hide_gtk)
      PUT_C("(GtkWidget *)");
    if (IS_GLOBAL(MENU_ITEM_SUBMENU(gw))) PUT_C("this->");
    PUT_C(get_identifier(WIDGET_NAME(MENU_ITEM_SUBMENU(gw))));
    PUT_C(");\n");
  }

  if (MENU_ITEM_GROUP(gw) != -1) {
    gtk_widget sw = MENU_ITEM_GROUP(gw);
    gtk_widget dum;
    do {
      dum = sw;
      sw = MENU_ITEM_GROUP(sw);
    } while (sw != -1);
    sw = dum;
    PUT_C("  gtk_radio_menu_item_set_group(GTK_RADIO_MENU_ITEM(");
    if (IS_GLOBAL(gw)) PUT_C("this->");
    PUT_C(get_identifier(WIDGET_NAME(gw)));
    PUT_C("),\n      gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(");
    if (IS_GLOBAL(sw)) PUT_C("this->");
    PUT_C(get_identifier(WIDGET_NAME(sw)));
    PUT_C(")));\n");
  }

  return 0;
}

/* -1 error, 0 ok */
static int generate_signals(gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);
  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    switch (id) {
      CASE_GENERATE_SIGNAL_MENU_ITEM_ACTIVATE
      CASE_GENERATE_SIGNAL_DEFAULT
    }
  }
  if (list_size(l)) PUT_C("\n");

  if ((MENU_ITEM_TYPE(gw) == MENU_ITEM_CHECK ||
       MENU_ITEM_TYPE(gw) == MENU_ITEM_RADIO)) {
    PUT_C("  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(");
    if (IS_GLOBAL(gw)) PUT_C("this->");
    PUT_C(get_identifier(WIDGET_NAME(gw)));
    if (IS_ACTIVE(gw))
      PUT_C("), TRUE);\n");
    else
      PUT_C("), FALSE);\n");
  }

  return 0;
}

widget_driver menu_item_widget_driver = {
  .name = "menu_item",
  .first_pass_translation = 0,
  .second_pass_translation = 0,
  .populate_properties = populate_properties,
  .generate_struct = generate_struct,
  .generate_signal_handlers = generate_signal_handlers,
  .generate_variable_declaration = generate_variable_declaration,
  .generate_variable_instanciation = generate_variable_instanciation,
  .generate_pack = generate_pack,
  .generate_post_pack = 0,
  .generate_signals = generate_signals
};
